/* UW8 - Operator Services src/misc_tools.c
 * Copyright (c) 1998 Chris Birch <uw8@othernet.org>
 * Original code and design by Michael Dabrowski <devon@othernet.org> UW7.5
 * All further GNU development and publication rights of this software are
 * granted to Chris Birch by Michael Dabrowski.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */           


#include<bot.h>

int check_level(char *who, char *cmd)
{
    int cmdlvl = 0;
    cmdlvl = get_command_level(cmd);
    if(cmdlvl > added_level)
    {
       send_denied_msg(who);
       return 0;
    }
    return 1;
}

void debug(char *format, ...)
{
#ifdef DEBUG
   char buff[500];
   va_list arglist;
   va_start(arglist, format);
   vsprintf(buff, format, arglist);
   if(*msg_chan == '#')
     send_to_server(":%s PRIVMSG %s :%s\n", bot.nick, msg_chan, buff);
   va_end(arglist);
#endif
}

char *date()
{
   time_t timer;
   time(&timer);
   strcpy(strbuff, ctime(&timer));
   strip_newline(strbuff);
   return strbuff;
}

char *bottime()
{
   time_t timer;
   struct tm *ntime;

   time(&timer);
   ntime=localtime(&timer);
   sprintf(strbuff, "%02d:%02d:%02d",ntime->tm_hour,ntime->tm_min,ntime->tm_sec);
   return (strbuff);
}

void command_log(char *who, char *cmd, char *rest)
{
   if(joined == TRUE)
   {
      if(!strcmp(cmd, "PRIVMSG")) return;
      if(rest == NULL)
      {
         log(LOG_FILE, "[%s] %s %s\n", date(), who, cmd);
         send_to_server(":%s PRIVMSG %s : [%s] %s by %s\n", botnick, msg_chan, bottime(), cmd, who);
      }
      else
      {
         log(LOG_FILE, "[%s] %s %s %s\n", date(), who, cmd, rest);
         send_to_server(":%s PRIVMSG %s : [%s] %s by %s\n", botnick, msg_chan, bottime(), cmd, who);
      }
   }
}

int send_to_server(char *format, ...)
{
   va_list arglist;
   char buffer[MAXBUF];
   buffer[0]='\0';
   va_start(arglist, format);
   vsprintf(buffer, format, arglist);
   va_end(arglist);
   return(writeln(buffer));
}

void send_denied_msg(char *nick)
{
   char *nwho = NULL;
   nwho = nick2num(nick);
   send_to_server(":%s NOTICE %s :%s\n", bot.nick, nwho, denied_msg);
}

char *get_command_name(char *command)
{
   int i;
   for(i = 0; commands[i].name; i++)
   {
      if(!strcasecmp(command, commands[i].realname)) return commands[i].name;
   }
   return;
}

int get_command_level(char *command)
{
   int i;
   for(i = 0; commands[i].name; i++)
   {
      if(!strcasecmp(command, commands[i].realname)) return commands[i].level;
   }
   return -1;
}

void file_error(char *fname)
{
   fprintf(stderr, "Error opening %s\n", fname);
  log(ERRLOG_FILE, "[%s] Error opening %s\n", date(), fname);
}

char *get_time(long ttime)
{
  char tmp[200];
  struct tm *tmptr;
  int isdst=0,gmtoffset;

  newtime[0] = '\0';
  tmptr=localtime(&ttime);
  strcpy(tmp,ctime(&ttime));
  strip_newline(tmp);

  if (tmptr->tm_isdst)
   isdst=1;
#ifdef BSD
  gmtoffset=(tmptr->tm_gmtoff/SECPERHR-isdst)*(-1);
  sprintf(newtime,"%s %s %d",tmp,tmptr->tm_zone,gmtoffset);
#else
  gmtoffset=(timezone/SECPERHR-isdst)*(-1);
  sprintf(newtime,"%s %s %d",tmp,tzname[isdst],gmtoffset);
#endif
  return(newtime);
}

void load_commands()
{
   FILE *fp;
   char buff[100], *command= NULL, *funct= NULL, *level= NULL, *disable= NULL, *rest= NULL, cmds_out[150];
   int i = 0;

   buff[0]='\0';cmds_out[0]='\0';
   strcpy(buff, COM_FILE);
   if((fp = fopen(buff, "r")) == NULL)
   {
      file_error(buff);
      return;
   }
   while(fgets(buff, 100, fp) != NULL)
   {
      i = 0;
      if(buff[0] == '*') continue;
      strip_newline(buff);
      if(funct = strchr(buff, ' ')) *funct++ = '\0';
      while(*funct == ' ') *funct++;
      if(level = strchr(funct, ' ')) *level++ = '\0';
      while(*level == ' ') *level++;
      if(disable = strchr(level, ' ')) *disable++ = '\0';
      while(*disable == ' ') *disable++;
      if(rest = strchr(disable, ' '));
      for(i = 0; *commands[i].name; i++)
      {
         if(!strcasecmp(commands[i].name, funct))
         {
            strcpy(commands[i].name, buff);
            strcpy(commands[i].realname, funct);
            commands[i].level = atoi(level);
            commands[i].disable = atoi(disable);
         }
      }
   }
   fclose(fp);
}

int secretchan(char *who, char *chan)
{
   struct secretinfo *channel;
   char buff[100], *host, *nwho;

   nwho = num2nick(who);
   channel = secrethead;
   strlwr(chan);
   while(channel != NULL)
   {
      strlwr(channel->channel);
      if(strstr(chan,channel->channel))
      {
          return -1;
      }
      channel=channel->next;
   }
   return 0;
}

int badchan(char *who, char *chan)
{
   struct badinfo *channel;
   char buff[100], *host, *nwho;

   nwho = num2nick(who);
   channel = badhead;
   strlwr(chan);
   while(channel != NULL)
   {
      strlwr(channel->channel);
      if(strstr(chan,channel->channel))
      {
          return -1;
      }
      channel=channel->next;
   }
   return 0;
}

int trusted(char *who, char *host)
{
   struct trustinfo *thost;
   char buff[100], *nwho;

   nwho = num2nick(who);
   thost = trusthead;
   while(thost != NULL)
   {
      if (!wcmp(thost->host, host))
      {
          return -1;
      }
      thost=thost->next;
   }
   return 0;
}

int count_users()
{
   struct nickinfo *nick;
   int i = 0;
   
   nick = nickhead;
   while(nick != NULL)
   {
      i++;
      nick = nick->next;
   }
   number = i;
   return(number);
}
